unit DirectSpeechRecognitionControlU;

//Uses the low level MS Direct Speech Recognition ActiveX (ActiveListen)
//to simplify coding and to provide animation

{$WRITEABLECONST On}
              
interface

uses
  Speech,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, OleCtrls, ACTIVELISTENPROJECTLib_TLB, ComCtrls;

type
  TfrmDirectSRControl = class(TForm)
    DirectSR: TDirectSR;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lstProgress: TListBox;
    GroupBox1: TGroupBox;
    lblMinVU: TLabel;
    lblMaxVU: TLabel;
    lblVU: TLabel;
    ProgressBar: TProgressBar;
    memText: TMemo;
    lstHypotheses: TListBox;
    btnAbout: TButton;
    btnGeneral: TButton;
    btnLexicon: TButton;
    btnTrainGeneral: TButton;
    btnTrainMic: TButton;
    procedure FormCreate(Sender: TObject);
    procedure DirectSRAttribChanged(Sender: TObject; Attribute: Integer);
    procedure DirectSRBookMark(Sender: TObject; MarkID: Integer);
    procedure DirectSRError(Sender: TObject; warning: Cardinal;
      const Details, Message: WideString);
    procedure DirectSRInterference(Sender: TObject; beginhi, beginlo,
      endhi, endlo, type_: Integer);
    procedure DirectSRPaused(Sender: TObject);
    procedure DirectSRPhraseFinish(Sender: TObject; flags, beginhi,
      beginlo, endhi, endlo: Integer; const Phrase, parsed: WideString;
      results: Integer);
    procedure DirectSRPhraseHypothesis(Sender: TObject; flags, beginhi,
      beginlo, endhi, endlo: Integer; const Phrase: WideString;
      results: Integer);
    procedure DirectSRPhraseStart(Sender: TObject; hi, lo: Integer);
    procedure DirectSRReEvaluate(Sender: TObject; results: Integer);
    procedure DirectSRSound(Sender: TObject; beginhi, beginlo, endhi,
      endlo: Integer);
    procedure DirectSRUtteranceBegin(Sender: TObject; beginhi,
      beginlo: Integer);
    procedure DirectSRUtteranceEnd(Sender: TObject; beginhi, beginlo,
      endhi, endlo: Integer);
    procedure DirectSRVUMeter(Sender: TObject; beginhi, beginlo,
      level: Integer);
    procedure DirectSRwarning(Sender: TObject; warning: Cardinal;
      const Details, Message: WideString);
    procedure btnAboutClick(Sender: TObject);
    procedure btnGeneralClick(Sender: TObject);
    procedure btnLexiconClick(Sender: TObject);
    procedure btnTrainGeneralClick(Sender: TObject);
    procedure btnTrainMicClick(Sender: TObject);
  public
    procedure Log(const Msg: String); overload;
    procedure Log(const Msg: String; const Args: array of const); overload;
  end;

var
  frmDirectSRControl: TfrmDirectSRControl;

implementation

{$R *.dfm}

procedure TfrmDirectSRControl.Log(const Msg: String);
begin
  if not Assigned(lstProgress) then
    Exit;
  lstProgress.Items.Add(Msg);
  lstProgress.ItemIndex := lstProgress.Items.Count - 1
end;

procedure TfrmDirectSRControl.Log(const Msg: String; const Args: array of const);
begin
  Log(Format(Msg, Args))
end;

procedure TfrmDirectSRControl.FormCreate(Sender: TObject);
begin
  DirectSR.GrammarFromString('[Grammar]'#13'LangID=2057'#13'Type=dictation'#13);
  DirectSR.Activate;
  //Need to set up a speaker to show the training dialog
  DirectSR.Speaker := 'blong';
end;

procedure TfrmDirectSRControl.DirectSRAttribChanged(Sender: TObject;
  Attribute: Integer);
var
  S: String;
begin
  if Attribute <> 0 then
  begin
    if Attribute and IVCNSAC_AUTOGAINENABLE > 0 then
      S := 'automatic gain, ';
    if Attribute and IVCNSAC_ENABLED > 0 then
      S := S + 'enabled, ';
    if Attribute and IVCNSAC_AWAKE > 0 then
      S := S + 'awake, ';
    if Attribute and IVCNSAC_DEVICE > 0 then
      S := S + 'audio device, ';
    if Attribute and IVCNSAC_MICROPHONE > 0 then
      S := S + 'current microphone, ';
    if Attribute and IVCNSAC_SPEAKER > 0 then
      S := S + 'speaker, ';
    if Attribute and IVCNSAC_SRMODE > 0 then
      S := S + 'SR mode, ';
    if Attribute and IVCNSAC_THRESHOLD > 0 then
      S := S + 'threshold, ';
    if Attribute and IVCNSAC_ORIGINAPP > 0 then
      S := S + 'from this app';
  end
  else
    S := 'none';
  Log('OnAttribChanged: ' + S)
end;

procedure TfrmDirectSRControl.DirectSRBookMark(Sender: TObject; MarkID: Integer);
begin
  Log('OnBookmark: %d', [MarkID])
end;

procedure TfrmDirectSRControl.DirectSRError(Sender: TObject; warning: Cardinal;
  const Details, Message: WideString);
begin
  Log('OnError: details = %s, Msg = %s', [Details, Message])
end;

procedure TfrmDirectSRControl.DirectSRInterference(Sender: TObject; beginhi, beginlo,
  endhi, endlo, type_: Integer);
var
  S: String;
begin
  case type_ of
    SRMSGINT_NOISE:              S := 'background noise too high';
    SRMSGINT_NOSIGNAL:           S := 'engine cannot detect a signal (mic unplugged?)';
    SRMSGINT_TOOLOUD:            S := 'speaker is too loud; recognition results may be degraded';
    SRMSGINT_TOOQUIET:           S := 'speaker is too quiet; recognition results may be degraded';
    SRMSGINT_AUDIODATA_STOPPED,
    SRMSGINT_IAUDIO_STOPPED:     S := 'engine has stopped receiving audio data from the audio source';
    SRMSGINT_AUDIODATA_STARTED,
    SRMSGINT_IAUDIO_STARTED:     S := 'engine has resumed receiving audio data from the audio source';
  else
    S := Format('type %d', [type_])
  end;
  Log('OnInterference: %s', [S])
end;

procedure TfrmDirectSRControl.DirectSRPaused(Sender: TObject);
begin
  Log('OnPaused')
end;

procedure TfrmDirectSRControl.DirectSRPhraseFinish(Sender: TObject; flags, beginhi,
  beginlo, endhi, endlo: Integer; const Phrase, parsed: WideString;
  results: Integer);
begin
  if flags and ISRNOTEFIN_RECOGNIZED > 0 then
  begin
    Log('OnPhraseFinish: %s', [phrase]);
    memText.SelText := phrase + #32;
  end
  else
    Log('OnPhraseFinish: unrecognised utterance');
end;

//This event doesn't actually fire
procedure TfrmDirectSRControl.DirectSRPhraseHypothesis(Sender: TObject; flags, beginhi,
  beginlo, endhi, endlo: Integer; const Phrase: WideString;
  results: Integer);
begin
  if flags and ISRNOTEFIN_RECOGNIZED > 0 then
  begin
    lstHypotheses.Items.Add(phrase);
    lstHypotheses.ItemIndex := lstHypotheses.Items.Count - 1
  end
  else
    Log('OnPhraseHypothesis: unrecognised phrase');
end;

procedure TfrmDirectSRControl.DirectSRPhraseStart(Sender: TObject; hi, lo: Integer);
begin
  Log('OnPhraseStart');
  lstHypotheses.Clear;
end;

procedure TfrmDirectSRControl.DirectSRReEvaluate(Sender: TObject; results: Integer);
begin
  Log('OnReEvaluate')
end;

procedure TfrmDirectSRControl.DirectSRSound(Sender: TObject; beginhi, beginlo, endhi,
  endlo: Integer);
begin
  Log('OnSound')
end;

procedure TfrmDirectSRControl.DirectSRUtteranceBegin(Sender: TObject; beginhi,
  beginlo: Integer);
begin
  Log('OnUtteranceBegin')
end;

procedure TfrmDirectSRControl.DirectSRUtteranceEnd(Sender: TObject; beginhi, beginlo,
  endhi, endlo: Integer);
begin
  Log('OnUtteranceEnd')
end;

procedure TfrmDirectSRControl.DirectSRVUMeter(Sender: TObject; beginhi, beginlo,
  level: Integer);
begin
  ProgressBar.Position := level;
  lblVU.Caption := IntToStr(level);
end;

procedure TfrmDirectSRControl.DirectSRwarning(Sender: TObject; warning: Cardinal;
  const Details, Message: WideString);
begin
  Log('OnWarning: details = %s, Msg = %s', [Details, Message])
end;

procedure TfrmDirectSRControl.btnAboutClick(Sender: TObject);
begin
  DirectSR.AboutDlg(Handle, '')
end;

procedure TfrmDirectSRControl.btnGeneralClick(Sender: TObject);
begin
  DirectSR.GeneralDlg(Handle, '')
end;

procedure TfrmDirectSRControl.btnLexiconClick(Sender: TObject);
begin
  DirectSR.LexiconDlg(Handle, '')
end;

procedure TfrmDirectSRControl.btnTrainGeneralClick(Sender: TObject);
begin
  DirectSR.TrainGeneralDlg(Handle, '')
end;

procedure TfrmDirectSRControl.btnTrainMicClick(Sender: TObject);
begin
  DirectSR.TrainMicDlg(Handle, '')
end;

end.
