unit DirectSpeechRecognitionAPIU;

//Uses the JEDI Speech import unit and the low-Level DirectSpeechRecognition API

interface

uses
  Speech,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls;

type
  TfrmDirectSRAPI = class(TForm)
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    lstProgress: TListBox;
    GroupBox1: TGroupBox;
    lblMinVU: TLabel;
    lblMaxVU: TLabel;
    lblVU: TLabel;
    ProgressBar: TProgressBar;
    memText: TMemo;
    lstHypotheses: TListBox;
    btnAbout: TButton;
    btnGeneral: TButton;
    btnLexicon: TButton;
    btnTrainGeneral: TButton;
    btnTrainMic: TButton;
    Label4: TLabel;
    Label6: TLabel;
    cbEngines: TComboBox;
    lstEngineInfo: TListBox;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbEnginesChange(Sender: TObject);
    procedure btnAboutClick(Sender: TObject);
    procedure btnGeneralClick(Sender: TObject);
    procedure btnLexiconClick(Sender: TObject);
    procedure btnTrainGeneralClick(Sender: TObject);
    procedure btnTrainMicClick(Sender: TObject);
  private
    AMMD: IAudioMultiMediaDevice;
    SREnum: ISREnum;
    PModeInfo: PSRModeInfo; //mode info for selected engine
    SRCentral: ISRCentral;
    SRDialogs: ISRDialogs;
    SRAttrs: ISRAttributes;
    EngineSinkCookie: DWord;
    SRGramCommon: ISRGramCommon;
  public
    procedure Log(const Msg: String); overload;
    procedure Log(const Msg: String; const Args: array of const); overload;
  end;

  TSRNotifySink = class(TInterfacedObject, ISRNotifySink)
  private
    FForm: TfrmDirectSRAPI;
  protected
    function AttribChanged(dwAttribute: DWORD): HResult; stdcall;
    function Interference(qTimeStampBegin: QWORD; qTimeStampEnd: QWORD;
      dwType: DWORD): HResult; stdcall;
    function Sound(qTimeStampBegin: QWORD; qTimeStampEnd: QWORD): HResult; stdcall;
    function UtteranceBegin(qTimeStampBegin: QWORD): HResult; stdcall;
    function UtteranceEnd(qTimeStampBegin: QWORD; qTimeStampEnd: QWORD): HResult; stdcall;
    function VUMeter(qTimeStampBegin: QWORD; wLevel: WORD): HResult; stdcall;
  public
    constructor Create(Form: TfrmDirectSRAPI);
  end;

  TSRGramNotifySink = class(TInterfacedObject, ISRGramNotifySink)
  private
    FForm: TfrmDirectSRAPI;
    function PhraseToStr(pSRPhrase: PSRPhraseA): String;
  protected
    function BookMark(dwBookMarkID: DWORD): HResult; stdcall;
    function Paused: HResult; stdcall;
    function PhraseFinish(dwFlags: DWORD; qTimeStampBegin: QWORD;
      qTimeStampEnd: QWORD; pSRPhrase: PSRPhrase; IUnknownResult: IUnknown): HResult; stdcall;
    function PhraseHypothesis(dwFlags: DWORD; qTimeStampBegin: QWORD;
      qTimeStampEnd: QWORD; pSRPhrase: PSRPhrase; IUnknownResult: IUnknown): HResult; stdcall;
    function PhraseStart(qTimeStampBegin: QWORD): HResult; stdcall;
    function ReEvaluate(IUnk: IUnknown): HResult; stdcall;
    function Training(dwTrain: DWORD): HResult; stdcall;
    function UnArchive(IUnknownResult: IUnknown): HResult; stdcall;
  public
    constructor Create(Form: TfrmDirectSRAPI);
  end;

var
  frmDirectSRAPI: TfrmDirectSRAPI;

implementation

{$R *.dfm}

uses
  ActiveX, ComObj, MMSystem;

{$ifndef CONDITIONALEXPRESSIONS}
type
  PPWideChar = ^PWideChar;

{$endif}

procedure TfrmDirectSRAPI.Log(const Msg: String);
begin
  if not Assigned(lstProgress) then
    Exit;
  lstProgress.Items.Add(Msg);
  lstProgress.ItemIndex := lstProgress.Items.Count - 1
end;

procedure TfrmDirectSRAPI.Log(const Msg: String; const Args: array of const);
begin
  Log(Format(Msg, Args))
end;

{.$define SEE_GRAMMAR_COMPILER_MSG}
procedure TfrmDirectSRAPI.FormCreate(Sender: TObject);
var
  ModeInfo: TSRModeInfo;
  NumFound: DWord;
  STGramComp: ISTGramComp;
{$ifdef SEE_GRAMMAR_COMPILER_MSG}
  Size: DWord;
  GramErr: PWideChar;
{$endif}
  SRGramUnk: IUnknown;
  SRNotifySink: ISRNotifySink;
  SRGramNotifySink: ISRGramNotifySink;
const
  Grammar: PChar = '[Grammar]'#13'LangID=2057'#13'Type=dictation'#13;
begin
  SendMessage(lstProgress.Handle, LB_SETHORIZONTALEXTENT, Width, 0);
  Log('About to connect to multimedia device');
  AMMD := CreateComObject(CLSID_MMAudioSource) as IAudioMultiMediaDevice;
  OleCheck(AMMD.DeviceNumSet(WAVE_MAPPER));
  Log('Connected to multimedia device');
  Log('About to enumerate speech engines');
  SREnum := CreateComObject(CLSID_SREnumerator) as ISREnum;
  OleCheck(SREnum.Reset);
  OleCheck(SREnum.Next(1, ModeInfo, @NumFound));
  While NumFound > 0 do
  begin
    New(PModeInfo);
    PModeInfo^ := ModeInfo;
    cbEngines.Items.AddObject(
      String(ModeInfo.szModeName), TObject(PModeInfo));
    OleCheck(SREnum.Next(1, ModeInfo, @NumFound));
  end;
  if cbEngines.Items.Count > 0 then
  begin
    cbEngines.ItemIndex := 0; //Select 1st engine
    cbEngines.OnChange(cbEngines); //& ensure OnChange triggers
  end;
  Log('Enumerated speech engines');
  Log('About to start engine');
  OleCheck(SREnum.Select(PModeInfo.gModeID, SRCentral, AMMD));
  Log('Engine started');
  Log('About to make dialogs available');
  if PModeInfo.dwInterfaces and SRI_ISRDIALOGS > 0 then
  begin
    SRDialogs := SRCentral as ISRDialogs;
    Log('Dialogs are available');
  end
  else
    Log('Dialogs are not supported');
  Log('About to make speech attributes available');
  if PModeInfo.dwInterfaces and SRI_ISRATTRIBUTES > 0 then
  begin
    SRAttrs := SRCentral as ISRAttributes;
    Log('Attributes available');
  end
  else
    Log('Attributes interface not supported');
  //Load grammar
  STGramComp := CreateComObject(CLSID_STGramComp) as ISTGramComp;
  OleCheck(STGramComp.FromMemory(Grammar, Succ(StrLen(Grammar))));
  //Compile grammar
{$ifdef SEE_GRAMMAR_COMPILER_MSG}
  GramErr := nil;
  OleCheck(STGramComp.Compile(GramErr, @Size));
  if Assigned(GramErr) then
  begin
    ShowMessage(WideString(GramErr));
    CoTaskMemFree(GramErr);
  end;
{$else}
  OleCheck(STGramComp.Compile(PPWideChar(nil)^, nil));
{$endif}

  //Start a dictation session
  SRGramNotifySink := TSRGramNotifySink.Create(Self);
  OleCheck(STGramComp.GrammarLoad(SRCentral, Pointer(SRGramNotifySink),
    ISRGramNotifySink, SRGramUnk));
  SRNotifySink := TSRNotifySink.Create(Self);
  OleCheck(SRCentral.Register(Pointer(SRNotifySink), ISRNotifySink,
    EngineSinkCookie));
  OleCheck(SRAttrs.SpeakerSet('blong'));
  if Supports(SRGramUnk, ISRGramCommon, SRGramCommon) then
    OleCheck(SRGramCommon.Activate(Handle, False, nil))
end;

procedure TfrmDirectSRAPI.FormDestroy(Sender: TObject);
var
  I: Integer;
begin
  OleCheck(SRCentral.UnRegister(EngineSinkCookie));
  //Clean up TSRModeInfo memory for each speech engine
  for I := 0 to cbEngines.Items.Count - 1 do
    Dispose(PSRModeInfo(cbEngines.Items.Objects[I]));
end;

procedure TfrmDirectSRAPI.cbEnginesChange(Sender: TObject);
const
  Sequences: array[SRSEQUENCE_DISCRETE..SRSEQUENCE_CONTCFGDISCDICT]
    of String = (
      'recognises discrete utterances',
      'performs continuous recognition',
      'performs word spotting',
      'pPerforms continuous recognition for CFGs and discrete dictation');
begin
  with lstEngineInfo.Items do
  begin
    Clear;
    PModeInfo := PSRModeInfo(cbEngines.Items.Objects[cbEngines.ItemIndex]);
    Add(Format('Name:'#9'%s', [PModeInfo.szModeName]));
    Add(Format('Language:'#9'%s', [PModeInfo.Language.szDialect]));
    Add(Format('Manufacturer:'#9'%s', [PModeInfo.szMfgName]));
    Add(Format('Product:'#9'%s', [PModeInfo.szProductName]));
    Add(Format('Mode ID:'#9'%s', [GUIDToString(PModeInfo.gModeID)]));
    Add(Format('Engine ID:'#9'%s', [GUIDToString(PModeInfo.gEngineID)]));
    Add(Format('Sequencing:'#9'%s', [Sequences[PModeInfo.dwSequencing]]));
  end
end;

procedure TfrmDirectSRAPI.btnAboutClick(Sender: TObject);
begin
  if Assigned(SRDialogs) then
    OleCheck(SRDialogs.AboutDlg(Handle, nil))
end;

procedure TfrmDirectSRAPI.btnGeneralClick(Sender: TObject);
begin
  if Assigned(SRDialogs) then
    OleCheck(SRDialogs.GeneralDlg(Handle, nil))
end;

procedure TfrmDirectSRAPI.btnLexiconClick(Sender: TObject);
begin
  if Assigned(SRDialogs) then
    OleCheck(SRDialogs.LexiconDlg(Handle, nil))
end;

procedure TfrmDirectSRAPI.btnTrainGeneralClick(Sender: TObject);
begin
  if Assigned(SRDialogs) then
    OleCheck(SRDialogs.TrainGeneralDlg(Handle, nil))
end;

procedure TfrmDirectSRAPI.btnTrainMicClick(Sender: TObject);
begin
  if Assigned(SRDialogs) then
    OleCheck(SRDialogs.TrainMicDlg(Handle, nil))
end;

{ TSRNotifySink }

constructor TSRNotifySink.Create(Form: TfrmDirectSRAPI);
begin
  inherited Create;
  FForm := Form
end;

function TSRNotifySink.AttribChanged(dwAttribute: DWORD): HResult;
var
  S: String;
begin
  Result := S_OK;
  if dwAttribute <> 0 then
  begin
    if dwAttribute and IVCNSAC_AUTOGAINENABLE > 0 then
      S := 'automatic gain, ';
    if dwAttribute and IVCNSAC_ENABLED > 0 then
      S := S + 'enabled, ';
    if dwAttribute and IVCNSAC_AWAKE > 0 then
      S := S + 'awake, ';
    if dwAttribute and IVCNSAC_DEVICE > 0 then
      S := S + 'audio device, ';
    if dwAttribute and IVCNSAC_MICROPHONE > 0 then
      S := S + 'current microphone, ';
    if dwAttribute and IVCNSAC_SPEAKER > 0 then
      S := S + 'speaker, ';
    if dwAttribute and IVCNSAC_SRMODE > 0 then
      S := S + 'SR mode, ';
    if dwAttribute and IVCNSAC_THRESHOLD > 0 then
      S := S + 'threshold, ';
    if dwAttribute and IVCNSAC_ORIGINAPP > 0 then
      S := S + 'from this app';
  end
  else
    S := 'none';
  FForm.Log('Attribute changed: ' + S)
end;

function TSRNotifySink.Interference(qTimeStampBegin, qTimeStampEnd: QWORD;
  dwType: DWORD): HResult;
var
  S: String;
begin
  Result := S_OK;
  case dwType of
    SRMSGINT_NOISE:              S := 'background noise too high';
    SRMSGINT_NOSIGNAL:           S := 'engine cannot detect a signal (mic unplugged?)';
    SRMSGINT_TOOLOUD:            S := 'speaker is too loud; recognition results may be degraded';
    SRMSGINT_TOOQUIET:           S := 'speaker is too quiet; recognition results may be degraded';
    SRMSGINT_AUDIODATA_STOPPED,
    SRMSGINT_IAUDIO_STOPPED:     S := 'engine has stopped receiving audio data from the audio source';
    SRMSGINT_AUDIODATA_STARTED,
    SRMSGINT_IAUDIO_STARTED:     S := 'engine has resumed receiving audio data from the audio source';
  else
    S := Format('type %d', [dwType])
  end;
  FForm.Log('Interference: %s', [S])
end;

function TSRNotifySink.Sound(qTimeStampBegin,
  qTimeStampEnd: QWORD): HResult;
begin
  Result := S_OK;
  FForm.Log('Sound')
end;

function TSRNotifySink.UtteranceBegin(qTimeStampBegin: QWORD): HResult;
begin
  Result := S_OK;
  FForm.Log('UtteranceBegin');
end;

function TSRNotifySink.UtteranceEnd(qTimeStampBegin,
  qTimeStampEnd: QWORD): HResult;
begin
  Result := S_OK;
  FForm.Log('UtteranceEnd');
end;

function TSRNotifySink.VUMeter(qTimeStampBegin: QWORD;
  wLevel: WORD): HResult;
begin
  Result := S_OK;
  FForm.ProgressBar.Position := wLevel;
  FForm.lblVU.Caption := IntToStr(wLevel)
end;

{ TSRGramNotifySink }

constructor TSRGramNotifySink.Create(Form: TfrmDirectSRAPI);
begin
  inherited Create;
  FForm := Form
end;

function TSRGramNotifySink.BookMark(dwBookMarkID: DWORD): HResult;
begin
  Result := S_OK;
  FForm.Log('BookMark: %d', [dwBookMarkID])
end;

function TSRGramNotifySink.Paused: HResult;
begin
  Result := S_OK;
  FForm.Log('Paused')
end;

function TSRGramNotifySink.PhraseToStr(pSRPhrase: PSRPhraseA): String;
var
  ToGo: Integer;
  PSRW: PSRWord;
begin
  Result := '';
  if pSRPhrase = nil then
    Exit;
  ToGo := pSRPhrase.dwSize - SizeOf(pSRPhrase.dwSize);
  PSRW := @pSRPhrase.abWords;
  while ToGo > 0 do
  begin
    Result := Result + PChar(@PSRW.szWord) + #32;
    Dec(ToGo, PSRW.dwSize);
    Inc(PChar(PSRW), PSRW.dwSize)
  end;
end;

function TSRGramNotifySink.PhraseFinish(dwFlags: DWORD; qTimeStampBegin,
  qTimeStampEnd: QWORD; pSRPhrase: PSRPhrase;
  IUnknownResult: IUnknown): HResult;
begin
  Result := S_OK;
  if dwFlags and ISRNOTEFIN_RECOGNIZED > 0 then
  begin
    FForm.Log('PhraseFinish: %s', [PhraseToStr(pSRPhrase)]);
    FForm.memText.SelText := PhraseToStr(pSRPhrase);
  end
  else
    FForm.Log('PhraseFinish: unrecognised phrase')
end;

function TSRGramNotifySink.PhraseHypothesis(dwFlags: DWORD;
  qTimeStampBegin, qTimeStampEnd: QWORD; pSRPhrase: PSRPhrase;
  IUnknownResult: IUnknown): HResult;
begin
  Result := S_OK;
  if dwFlags and ISRNOTEFIN_RECOGNIZED > 0 then
  begin
    FForm.lstHypotheses.Items.Add(PhraseToStr(pSRPhrase));
    FForm.lstHypotheses.ItemIndex := FForm.lstHypotheses.Items.Count - 1
  end
  else
    FForm.Log('PhraseHypothesis: unrecognised phrase')
end;

function TSRGramNotifySink.PhraseStart(qTimeStampBegin: QWORD): HResult;
begin
  Result := S_OK;
  FForm.lstHypotheses.Items.Clear;
  FForm.Log('PhraseStart')
end;

function TSRGramNotifySink.ReEvaluate(IUnk: IUnknown): HResult;
begin
  Result := S_OK;
  FForm.Log('ReEvaluate')
end;

function TSRGramNotifySink.Training(dwTrain: DWORD): HResult;
var
  S: String;
const
  SRGNSTTRAIN_MICROPHONE = 1;
  SRGNSTTRAIN_GRAMMAR    = 2;
  SRGNSTTRAIN_GENERAL    = 3;
begin
  Result := S_OK;
  if dwTrain <> 0 then
    case dwTrain of
      SRGNSTTRAIN_MICROPHONE: S := 'for the current microphone';
      SRGNSTTRAIN_GRAMMAR: S := 'for the current grammar';
      SRGNSTTRAIN_GENERAL: S := 'in general';
    end;
  FForm.Log('Training required %s', [S])
end;

function TSRGramNotifySink.UnArchive(IUnknownResult: IUnknown): HResult;
begin
  Result := S_OK;
  FForm.Log('UnArchive')
end;

end.
