unit VoiceDictationControlU;

//Uses the high level Microsoft Voice Dictation ActiveX to simplify coding

interface

uses
  Speech,
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, OleCtrls, DICTLib_TLB, ComCtrls;

type
  TfrmVoiceDictationControl = class(TForm)
    lstProgress: TListBox;
    GroupBox1: TGroupBox;
    ProgressBar: TProgressBar;
    lblMinVU: TLabel;
    lblMaxVU: TLabel;
    lblVU: TLabel;
    memText: TMemo;
    lstHypotheses: TListBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Vdict: TVdict;
    btnAbout: TButton;
    btnGeneral: TButton;
    btnLexicon: TButton;
    btnTrainGeneral: TButton;
    btnTrainMic: TButton;
    procedure FormCreate(Sender: TObject);
    procedure VdictAttribChanged(Sender: TObject; Attrib: Integer);
    procedure VdictCommandBuiltIn(Sender: TObject;
      const command: WideString);
    procedure VdictCommandOther(Sender: TObject;
      const command: WideString);
    procedure VdictCommandRecognize(Sender: TObject; Id, flags: Integer;
      const action, command: WideString);
    procedure VdictDictating(Sender: TObject; const appstring: WideString;
      flags: Integer);
    procedure VdictGlobalKey(Sender: TObject; wParam, lParam: Integer);
    procedure VdictGlobalMouse(Sender: TObject; wParam, lParam, x,
      y: Integer);
    procedure VdictInterference(Sender: TObject; Attrib: Integer);
    procedure VdictPhraseFinish(Sender: TObject; flags: Integer;
      const phrase: WideString);
    procedure VdictPhraseHypothesis(Sender: TObject; flags: Integer;
      const phrase: WideString);
    procedure VdictPhraseStart(Sender: TObject);
    procedure VdictTextBookmarkChanged(Sender: TObject; val: Integer);
    procedure VdictTextChanged(Sender: TObject; reason: Integer);
    procedure VdictTextSelChanged(Sender: TObject);
    procedure VdictTraining(Sender: TObject; train: Integer);
    procedure VdictUtteranceBegin(Sender: TObject);
    procedure VdictUtteranceEnd(Sender: TObject);
    procedure VdictVUMeter(Sender: TObject; vu: Integer);
    procedure btnAboutClick(Sender: TObject);
    procedure btnGeneralClick(Sender: TObject);
    procedure btnLexiconClick(Sender: TObject);
    procedure btnTrainGeneralClick(Sender: TObject);
    procedure btnTrainMicClick(Sender: TObject);
  public
    PhraseDone: Boolean;
    procedure Log(const Msg: String); overload;
    procedure Log(const Msg: String; const Args: array of const); overload;
  end;

var
  frmVoiceDictationControl: TfrmVoiceDictationControl;

implementation

{$R *.dfm}

uses
  ComObj, ActiveX;

{$ifndef CONDITIONALEXPRESSIONS}
function BoolToStr(B: Boolean; UseBoolStrs: Boolean = False): string;
const
  cSimpleBoolStrs: array[Boolean] of String = ('0', '-1');
  cBoolStrs: array[Boolean] of String = ('False', 'True');
begin
  if UseBoolStrs then
    Result := cBoolStrs[B]
  else
    Result := cSimpleBoolStrs[B];
end;
{$endif}

{ TfrmVoiceDictationAPI }

procedure TfrmVoiceDictationControl.Log(const Msg: String);
begin
  if not Assigned(lstProgress) then
    Exit;
  lstProgress.Items.Add(Msg);
  lstProgress.ItemIndex := lstProgress.Items.Count - 1
end;

procedure TfrmVoiceDictationControl.Log(const Msg: String; const Args: array of const);
begin
  Log(Format(Msg, Args))
end;

procedure TfrmVoiceDictationControl.FormCreate(Sender: TObject);
begin
  //Choose a speaker, so the training wizard works okay
  Vdict.Speaker := 'blong';
  Vdict.Mode := VSRMODE_CMDANDDCT;
  Vdict.ActivateAndAssignWindow(Handle);
end;

procedure TfrmVoiceDictationControl.VdictAttribChanged(Sender: TObject;
  Attrib: Integer);
var
  S: String;
begin
  if Attrib <> 0 then
  begin
    if Attrib and IVCNSAC_AUTOGAINENABLE > 0 then
      S := 'automatic gain, ';
    if Attrib and IVCNSAC_ENABLED > 0 then
      S := S + 'enabled, ';
    if Attrib and IVCNSAC_AWAKE > 0 then
      S := S + 'awake, ';
    if Attrib and IVCNSAC_DEVICE > 0 then
      S := S + 'audio device, ';
    if Attrib and IVCNSAC_MICROPHONE > 0 then
      S := S + 'current microphone, ';
    if Attrib and IVCNSAC_SPEAKER > 0 then
      S := S + 'speaker, ';
    if Attrib and IVCNSAC_SRMODE > 0 then
      S := S + 'SR mode, ';
    if Attrib and IVCNSAC_THRESHOLD > 0 then
      S := S + 'threshold, ';
    if Attrib and IVCNSAC_ORIGINAPP > 0 then
      S := S + 'from this app';
  end
  else
    S := 'none';
  Log('OnAttribChanged: ' + S)
end;

procedure TfrmVoiceDictationControl.VdictCommandBuiltIn(Sender: TObject;
  const command: WideString);
begin
  Log('OnCommandBuiltIn: %s', [command])
end;

procedure TfrmVoiceDictationControl.VdictCommandOther(Sender: TObject;
  const command: WideString);
begin
  Log('OnCommandOther: %s', [command])
end;

procedure TfrmVoiceDictationControl.VdictCommandRecognize(Sender: TObject;
  Id, flags: Integer; const action, command: WideString);
begin
  Log('OnCommandRecognize: cmd = %s, id = %d', [command, Id])
end;

procedure TfrmVoiceDictationControl.VdictDictating(Sender: TObject;
  const appstring: WideString; flags: Integer);
begin
  Log('OnDictating: %s, app = %s', [BoolToStr(Bool(flags), True), appstring])
end;

procedure TfrmVoiceDictationControl.VdictGlobalKey(Sender: TObject; wParam,
  lParam: Integer);
begin
  Log('OnGlobalKey: virtual key code $%x, info $%x', [wParam, lParam])
end;

procedure TfrmVoiceDictationControl.VdictGlobalMouse(Sender: TObject;
  wParam, lParam, x, y: Integer);
begin
  Log('OnGlobalMouse (%d,%d): msg: $%x', [x, y, wParam])
end;

procedure TfrmVoiceDictationControl.VdictInterference(Sender: TObject;
  Attrib: Integer);
var
  S: String;
begin
  case Attrib of
    SRMSGINT_NOISE:              S := 'background noise too high';
    SRMSGINT_NOSIGNAL:           S := 'engine cannot detect a signal (mic unplugged?)';
    SRMSGINT_TOOLOUD:            S := 'speaker is too loud; recognition results may be degraded';
    SRMSGINT_TOOQUIET:           S := 'speaker is too quiet; recognition results may be degraded';
    SRMSGINT_AUDIODATA_STOPPED,
    SRMSGINT_IAUDIO_STOPPED:     S := 'engine has stopped receiving audio data from the audio source';
    SRMSGINT_AUDIODATA_STARTED,
    SRMSGINT_IAUDIO_STARTED:     S := 'engine has resumed receiving audio data from the audio source';
  else
    S := Format('type %d', [Attrib])
  end;
  Log('OnInterference: %s', [S])
end;

procedure TfrmVoiceDictationControl.VdictPhraseFinish(Sender: TObject;
  flags: Integer; const phrase: WideString);
begin
  if flags and ISRNOTEFIN_RECOGNIZED > 0 then
  begin
    Log('OnPhraseFinish: %s', [phrase]);
    memText.SelText := phrase + #32;
  end
  else
    Log('OnPhraseFinish: unrecognised utterance');
  //Since OnPhraseStart never seems to trigger, this will
  //clear the old hypotheses list on the first new hypothesis
  PhraseDone := True
end;

procedure TfrmVoiceDictationControl.VdictPhraseHypothesis(Sender: TObject;
  flags: Integer; const phrase: WideString);
begin
  //Since OnPhraseStart never seems to trigger, this will
  //clear the old hypotheses list on the first new hypothesis
  if PhraseDone then
  begin
    lstHypotheses.Clear;
    PhraseDone := False
  end;
  if flags and ISRNOTEFIN_RECOGNIZED > 0 then
  begin
    lstHypotheses.Items.Add(phrase);
    lstHypotheses.ItemIndex := lstHypotheses.Items.Count - 1
  end
  else
    Log('OnPhraseHypothesis: unrecognised phrase');
end;

//This event doesn't actually fire
procedure TfrmVoiceDictationControl.VdictPhraseStart(Sender: TObject);
begin
  Log('OnPhraseStart')
end;

procedure TfrmVoiceDictationControl.VdictTextBookmarkChanged(
  Sender: TObject; val: Integer);
begin
  Log('OnTextBookmarkChanged: %d', [val])
end;

procedure TfrmVoiceDictationControl.VdictTextChanged(Sender: TObject;
  reason: Integer);
var
  S: String;
const
  VDCT_TEXTMOVED = 8;
begin
  if Reason <> 0 then
  begin
    if Reason and VDCT_TEXTADDED > 0 then
      S := 'text added, ';
    if Reason and VDCT_TEXTREMOVED > 0 then
      S := 'text removed, ';
    if Reason and VDCT_TEXTREPLACED > 0 then
      S := 'text replaced, ';
    if Reason and VDCT_TEXTMOVED > 0 then
      S := 'text moved, ';
  end;
  Log('OnTextChanged: %s', [S])
end;

procedure TfrmVoiceDictationControl.VdictTextSelChanged(Sender: TObject);
begin
  Log('OnTextSelChanged')
end;

procedure TfrmVoiceDictationControl.VdictTraining(Sender: TObject;
  train: Integer);
var
  S: String;
const
  SRGNSTTRAIN_MICROPHONE = 1;
  SRGNSTTRAIN_GRAMMAR    = 2;
  SRGNSTTRAIN_GENERAL    = 3;
begin
  if train <> 0 then
    case train of
      SRGNSTTRAIN_MICROPHONE: S := 'for the current microphone';
      SRGNSTTRAIN_GRAMMAR: S := 'for the current grammar';
      SRGNSTTRAIN_GENERAL: S := 'in general';
    end;
  Log('OnTraining: training required %s', [S])
end;

procedure TfrmVoiceDictationControl.VdictUtteranceBegin(Sender: TObject);
begin
  Log('OnUtteranceBegin')
end;

procedure TfrmVoiceDictationControl.VdictUtteranceEnd(Sender: TObject);
begin
  Log('OnUtteranceEnd')
end;

procedure TfrmVoiceDictationControl.VdictVUMeter(Sender: TObject;
  vu: Integer);
begin
  ProgressBar.Position := vu;
  lblVU.Caption := IntToStr(vu);
end;

procedure TfrmVoiceDictationControl.btnAboutClick(Sender: TObject);
begin
  Vdict.AboutDlg(Handle, '');
end;

procedure TfrmVoiceDictationControl.btnGeneralClick(Sender: TObject);
begin
  Vdict.GeneralDlg(Handle, '');
end;

procedure TfrmVoiceDictationControl.btnLexiconClick(Sender: TObject);
begin
  Vdict.LexiconDlg(Handle, '');
end;

procedure TfrmVoiceDictationControl.btnTrainGeneralClick(Sender: TObject);
begin
  Vdict.TrainGeneralDlg(Handle, '');
end;

procedure TfrmVoiceDictationControl.btnTrainMicClick(Sender: TObject);
begin
  Vdict.TrainMicDlg(Handle, '');
end;

end.
